# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_br_curve_to_xy.m
# input: leng length up to which the x,y values are computed
#        A matrix of the type
#        A {a1  an}
#        where ai = [bi, ri]
#        up true for an upper and false for a lower curve
#       
# output x, y values of the curve
#
###########################################
function [x, y] = sla_br_curve_to_xy(leng, A, up)
  xval = 0 ;
  if (sla_check_br_curve(A, up))
    if (up)
      x(1) = 0 ;
      y(1) = 0 ;
      x(2) = 0 ;
      y(2) = A{1}(1) ;
      i = 3 ;
      while (xval < leng) 
        if (i <= columns(A)+1)
	  newx = (A{i-1}(1) - A{i-2}(1))/(A{i-2}(2)-A{i-1}(2)) ;
	  if (newx > leng)
	    newx = leng ;
          endif
	  x(i) = newx ;
	  y(i) = x(i) * A{i-2}(2) + A{i-2}(1) ;
          xval = newx ;
	  i++ ;
	else
	  x(i) = leng ;
          y(i) = x(i) * A{i-2}(2)  + A{i-2}(1) ;
	  xval = leng ;
	endif
      endwhile
    else
      x(1) = 0 ;
      y(1) = 0 ;
      x(2) = A{1}(1) ;
      y(2) = 0 ;
      i = 3 ;
      while (xval < leng) 
        if (i <= columns(A)+1)
	  newx = (A{i-1}(1)*A{i-1}(2) - A{i-2}(1)*A{i-2}(2))/(A{i-1}(2)-A{i-2}(2)) ;
	  if (newx > leng)
	    newx = leng ;
          endif
	  x(i) = newx ;
	  y(i) = (x(i)-A{i-2}(1)) * A{i-2}(2) ;
          xval = newx ;
	  i++ ;
	else
	  x(i) = leng ;
          y(i) = (x(i)-A{i-2}(1)) * A{i-2}(2) ;
	  xval = leng ;
	endif
      endwhile
    endif
  else
    printf("Sorry, curve specification inconsistent\n") ;    
  endif
endfunction