# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_convex_lower.m
# input: A matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output B convex lower bound for A
#
#
###########################################
function B = sla_convex_lower(A)

   if (sla_check_curve(A))
      if (sla_is_convex(A))
         B = A ;
         return ;
      else
         i = columns(A) ;
	 xval = A{i}(1) ;
	 yval = A{i}(2) ;
	 smax = A{i}(3) ;
	 maxdiff = 0 ;
	 for (k=1:columns(A)-1)
	    diff = (yval - (xval-A{k}(1))*smax) - A{k}(2) ;
	    if (diff > maxdiff)
	       maxdiff = diff ;
	    endif
	    diff = (yval-(xval-A{k+1}(1))*smax) - (A{k}(2)+(A{k+1}(1)-A{k}(1))*A{k}(3)) ;
	    if (diff > maxdiff)
	       maxdiff = diff ;
	    endif
	 endfor;
	 yval -= maxdiff ;
	 if ((xval - A{1}(1)) * smax > yval - A{1}(2))
	    B = {[A{1}(1), A{1}(2), (yval-A{1}(2))/(xval-A{1}(1))]} ;
	    B{2} = [xval, yval, smax] ;
	 else
	    B = {[A{1}(1), A{1}(2), smax]} ;
	    return ;
	 endif
	 i-- ;
	 while (i > 1)
	    xmin = 0 ;
 	    ymin = 1.00e+12 ;
	    kmin = 0 ;
	    k = i ;
	    for (k=1:i)
	       xval = A{k}(1) ;
	       yval = A{k}(2) ;
	       if (yval < ymin && yval < (xval-B{1}(1))*B{1}(3)+B{1}(2))
	          ymin = yval ;
	          xmin = xval ;
		  kmin = k ;
	       endif
	       if (k > 1)
	          yval = (A{k}(1)-A{k-1}(1))*A{k-1}(3)+A{k-1}(2) ;
	       	  if (yval < ymin && yval < (xval-B{1}(1))*B{1}(3)+B{1}(2))
	             ymin = yval ;
	             xmin = xval ;
		     kmin = k ;
	       	  endif
	       endif
	    endfor
	    if (kmin == 0)
	       return ;
	    else
	       j = columns(B) ;
	       while (j > 1)
	          B{j+1} = B{j} ;
		  j-- ;
	       endwhile
	       sval = (B{3}(2)-ymin)/(B{3}(1)-xmin) ;
	       if (sval < smax)
	          B{2} = [xmin, ymin, sval] ;
		  B{1}(3) = (ymin-B{1}(2))/(xmin-B{1}(1)) ;
		  smax = sval ;
	       else
	          yval = B{3}(2) - (B{3}(1)-xmin)*smax ;
	          B{2} = [xmin, yval, smax] ;
		  B{1}(3) = (yval-B{1}(2)) / (xmin-B{1}(1));
	       endif
	       if (B{1}(3) < 0.0)
	          B{1}(2) = B{2}(2) ;
		  B{1}(3) = 0.0 ;
	       endif
	       i = kmin-1 ;
	    endif
	 endwhile
	 B = sla_reduce(B) ;
      endif
   else
      printf("Sorry not a valid curve, cannot compute convex lower bound\n") ;
   endif
endfunction