# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_delay_at_t.m
# input: AlphaL lower arrival bound
#        PhiL lower cummulated delay curve
#        PhiU upper cummulated delay curve
#       
# output EtaL lower delay function
#        EtaU upper delay function
#
# description: The function computes bounds for
#              the actual daly at time t (eq. (14) .
#
###########################################
function [EtaL, EtaU] = sla_delay_at_t(AlphaL, PhiL, PhiU)

   phl = sla_derivative(PhiL) ;
   phu = sla_derivative(PhiU) ;
   vl = [0, phl{1}(2), 0] ;
   vu = [0, phu{1}(2), 0] ;
   EtaL = {vl} ;
   EtaU = {vu} ;
   j = 2 ;
   for (i=1:columns(phl))
      x = sla_get_x(AlphaL, phl{i}(1)) ;
      if (x >= 0)
         vl = [x, phl{i}(2), 0] ;
         EtaL{j} = vl ;
         j++ ;
      else
         break ;
      endif
   endfor
   EtaL = sla_reduce(EtaL) ;
   j = 2 ;
   for (i=1:columns(phu))
      x = sla_get_x(AlphaL, phu{i}(1)) ;
      if (x >= 0)
         vu = [x, phu{i}(2), 0] ;
         EtaU{j} = vu ;
         j++ ;
      else
         break ;
      endif
   endfor
   EtaU = sla_reduce(EtaU) ;
endfunction