# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_delay_from_input_output.m
# input: Alpha arrival process
#        Gamma departure process
#       
# output Phi delay prcoess
#
# description: The function computes the
#              upper delay curve for a systemdelay curve from the
#              input and output process.
#              Results are exact for piecewise constant delays.
#
###########################################
function Phi = sla_delay_from_input_output(Alpha, Gamma)

    if (sla_is_zero(Alpha))
        Phi = {[0,0,0]} ;
    else
	GammaI = sla_invert(Gamma) ;
        AlphaI = sla_invert(Alpha) ;
    	Phi = sla_sub(sla_max(GammaI, AlphaI), AlphaI) ;
    	Phi = sla_reduce(sla_integrate(Phi)) ;
    endif
    
endfunction