# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_delay_from_service.m
# input: AlphaL, AlphaU input bounds 
#        SigmaL, SigmaU service bounds       
#       
# output  PhiL, PhiU delay bounds
#
# description: The function computes delay bounds from
#              servcie bounds by analyzing the systems
#              with minimal input - maximal service and
#              with maximal input - minimal service.
#
###########################################
function [PhiL, PhiU] = sla_delay_from_service(AlphaL, AlphaU, SigmaL, SigmaU)

   # The maximal delay results from the system with maximal input and
   # minimal service.
   PhiU = sla_upper_delay(AlphaU, SigmaL, SigmaL) ;
   # The minimal delay results from the system with minimal input and
   # maximal service.
   PhiL = sla_lower_delay(AlphaL, SigmaU, SigmaU) ;

endfunction