# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_is_smaller_or_equal.m
# input: A, B matrices of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output true if curve A is smaller or equal B
#
###########################################
function ok = sla_is_smaller_or_equal(A, B)

   if (sla_is_equal(A,B))
      ok = true ;
      return ;
   endif
   if (A{columns(A)}(3) -  B{columns(B)}(3) > 1.0e-12)
      ok = false ;
      return ;
   endif
   iA = 1 ;
   iB = 1 ;
   xmax = max (A{columns(A)}(1), B{columns(B)}(1)) + 1 ;
   [xA, yA] = sla_curve_to_xy(xmax, A) ;
   [xB, yB] = sla_curve_to_xy(xmax, B) ;
   while (true)
      if (abs(xA(iA)-xB(iB)) < 1.0e-12) 
         if (yA(iA) > yB(iB))
	    ok = false ;
     	    return ;
	 endif
	 x = xA(iA) ;
      else 
         if (xA(iA) < xB(iB)) 
	    x = xB(iB) ; 
	 else
	    x = xA(iA) ;
	 endif
	 valA = valB = -1.0 ;
	 if (iA < length(xA))
	    if (xA(iA+1) > xA(iA))
	       valA = (x-xA(iA))/(xA(iA+1)-xA(iA))*(yA(iA+1)-yA(iA)) + yA(iA) ;
            else
	       valA = yA(iA+1) ;
	    endif
	 else
	    valA = (x - xA(iA))*A{columns(A)}(3) + yA(iA) ;
	 endif
	 if (iB < length(xB))
	    if (xB(iB+1) > xB(iB))
	       valB = (x-xB(iB))/(xB(iB+1)-xB(iB))*(yB(iB+1)-yB(iB)) + yB(iB) ;
            else
	       valB = yB(iB+1) ;
	    endif
	 else
	    valB = (x - xB(iB))*B{columns(B)}(3) + yB(iB) ;
	 endif
	 if (valA > -1 && valB > -1 && valA > valB)
	    ok = false ;
	    return ;
	 endif
      endif
      if (iA == length(xA) && iB == length(xB))
         ok = true ;
	 return ;
      endif
      if (iA == length(xA))
         iB++ ;
      else
         if (iB == length(xB))
	    iA++ ;
         else
	    if (abs(xA(iA+1) - xB(iB+1)) < 1.0e-12)
	       iA++ ;
	       iB++ ;
            else
	       if (xA(iA+1) < xB(iB+1))
	          iA++ ;
	       else
	          iB++ ;
	       endif
	    endif
	 endif
      endif
   endwhile ;
endfunction