# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_min_closure
#
# input: A  matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output B = sub-additive closure of min_convolution of A
#            or an concave upper bound of this curve 
#        stat = 2 if nothing has been changed 0 if concave
#            upper bound is used 1 if closure has been computed
#
###########################################
function [B, status] = sla_min_closure(A)
  max_iter = 5 ;
  status = 2 ;
  iter = 0 ;
  AC = {[0,0,0]} ;
  AC = sla_min(A, AC) ;
  B = sla_min(A, sla_min_conv(A, A)) ;
  B = sla_reduce(B) ;
  while (iter < max_iter && !(sla_is_equal(AC, B)))
    AC = B ;
    B = sla_min(B, sla_min_conv(B, A)) ;
    iter++ ;
    [ok, B] = sla_test_cyclic(B) ;
    if (ok)
       printf("Found cyclic curve, substitute concave upper bound\n") ;
       status = 1 ;
       break ;
    endif
  endwhile
  if (iter == max_iter)
    printf("Did not find cyclic structure in %i iterations of min/plus convolution\n") ;
    printf("Substitute curve by concave upper bound\n") ;
    B = sla_concave_upper(B) ;
    status = 0 ;
  else
    if (iter > 1)
      status = 1 ;
    endif
  endif
  B = sla_reduce(B) ;

endfunction