# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_parallel_and.m
# input: Alpha1L, Alpha1U input bounds service 1
#        Phi1L, Phi1U delay bounds service 1
#        Alpha2L, Alpha2U input bounds service 1
#        Phi1L, Phi2U delay bounds service 1
#       
#       
# output AlphaL, AlphaU arrival bounds
#        PhiL, PhiU delay bounds 
#
# description: The function computes parameters
#              of a parallel and composition of servcies.
#
###########################################
function [AlphaL, AlphaU, PhiL, PhiU] = sla_parallel_and(Alpha1L, Alpha1U, Phi1L, Phi1U, Alpha2L, Alpha2U, Phi2L, Phi2U)

   # We start with the arrival bounds of the composed service
   AlphaL = sla_max(Alpha1L, Alpha2L) ;
   AlphaU = sla_min(Alpha1U, Alpha2U) ;
   # Compute the delay bounds
   PhiL = sla_max(Phi1L, Phi2L) ;
   PhiU = sla_max(Phi1U, Phi2U) ;
   AlphaL = sla_reduce(AlphaL) ;
   AlphaU = sla_reduce(AlphaU) ;
   PhiL = sla_reduce(PhiL) ;
   PhiU = sla_reduce(PhiU) ;

endfunction