# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_partial_min
#
# input: A matrix of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#        x1, y1, s1, x2, y2, s2, x3, y3 segements of aprtial curve
#        where -1 is interpreted as infinity
#
# output B = minimum of curve A and the segements defined by the
#            coordinates (x1,y1,s1)(x2, y2, s2) to (x3,y3) 
#
###########################################
function B = sla_partial_min(A, x1, y1, s1, x2, y2, s2, x3, y3)

   i = 1 ;
   B = {[0,0,0]} ;
   j = 2 ;
   # first the part until x1
   while (i <= columns(A))
      if (A{i}(1) < x1)
         B{j} = A{i} ;
	 i++ ;
         j++ ;
      else
         break ;
      endif
   endfor
   # treat the special case where the last segment of A starts
   # before x1 
   if (i > columns(A))
      B{j} = [x1, y1, s1] ;
      j++ ;
      B{j} = [x2, y2, s2] ;
      return ;
   endif
   # compute the part between x1 and x2
   if (i > 1)
      y = (x1 - A{i-1}(1))*A{i-1}(3) + A{i-1}(2) ;
      s = A{i-1}(3) ;
      if (y > y1)
         B{j} = [x1, y1, s1] ;
	 j++ ;
         if (s < s1)
	    z = (y1-y)/(s-s1) ;
	    if (z < x2 && z < A{i}(1))
 	       C{j} = [z, (z-x1)*s1, A{i-1}(3)] ;
	       j++ ;
	    endif
	 endif
      else
         if (s1 < s)
	    z = (y1-y)/(s-s1) ;
	    if (z < x2 && z < A{i}(1))
 	       B{j} = [z, (z-x1)*s1, s1] ;
	       j++ ;
	    endif
	 endif
      endif
   endif 
   while (i <= columns(A))
      if (A{i}(1) < x2)
         y = A{i}(2) ;
         s = A{i}(3) ;
	 yy1 = (A[{i}(1) - x1) * s1 + y1 ;
	 if (yy1 < y)
	    B{j} = [A{i}(1), yy1, s1) ;
	    j++ ;
	    if (s < s1)
	       z = (yy1-y)/(s-s1) ;
	       if (i < columns(A))
	       	  z2 = A{i+1}(1) ;
	       else
	          z2 = z ;
	       endif
	       if (z < x2 && z < z2)
	         B{j} = [z, (z-x1)*s1, s] ;
		 j++ ;  
	       endif
	    endif
	 else
	    B{j} = A{i} ;
            j++ ;
	    if (s1 < s)
	       z = (yy1-y)/(s-s1) ;
	       if (i < columns(A))
	       	  z2 = A{i+1}(1) ;
	       else
	          z2 = z ;
	       endif
	       if (z < x2 && z < z2)
	         B{j} = [z, (z-x1)*s1, s1] ; 
		 j++ ; 
	       endif
	    endif
	 endif
	 i++ ;
      else
         break ;
      endif
   endwhile
   # now compute between x2 and x3 



   # TO BE DONE
   A = sla_add_zero(A) ;
   B = sla_add_zero(B) ;
   if (A{1}(2) < B{1}(2))
      s = A{1}(3) ;
   else
      if (A{1}(2) > B{1}(2))
         s = B{1}(3) ;
      else
         s = min(A{1}(3), B{1}(3)) ;
      endif
   endif
   x = 0 ;
   y = min(A{1}(2),B{1}(2)) ;
   C = {[x, y, s]} ;
   i1 = 1 ;
   i2 = 1 ;
   j  = 2 ;
   while (true)
      x1 = A{i1}(1) ;
      x2 = B{i2}(1) ;
      s1 = A{i1}(3) ;
      s2 = B{i2}(3) ;
      y1 = (x - x1) * s1 + A{i1}(2) ;
      y2 = (x - x2) * s2 + B{i2}(2) ;
      if (i1 < columns(A))
         if (i2 < columns(B))
	    if (A{i1+1}(1) < B{i2+1}(1)) 
	       xnext = A{i1+1}(1) ;
	       ind = 1 ;
	    else
	       xnext = B{i2+1}(1) ;
	       ind = 2 ;
	    endif
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 else
	    xnext = A{i1+1}(1) ;
	    ind = 1 ;
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 endif
      else
         if (i2 < columns(B))
	    xnext = B{i2+1}(1) ;
	    ind = 2 ;
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 else
	    ind = 0 ;
	    xnext = 1.0e+12 ;
	 endif
      endif
      if ((y1 < y2 && s1 > s2) || (y1 > y2 && s1 < s2))
         xnew = (y1 -y2) /( s2 - s1) + x ;
	 ynew = y1 + s1*(xnew - x) ;
	 if (s1 < s2)
	    s = s1 ;
	 else
	    s = s2 ;
	 endif
	 if (xnew < xnext)
	    C{j} = [xnew, ynew, s] ;
	    j++ ;
	 endif
      endif
      if (ind == 0)
      	 C = sla_reduce(C) ;
         return ;
      endif
      if (ind == 1)
         i1++ ;
         y1next = A{i1}(2) ;	
	 s1 = A{i1}(3) ;
      else
         i2++ ;
	 y2next = B{i2}(2) ;
	 s2 = B{i2}(3) ;
      endif
      if (y1next < y2next)
      	 s = s1 ;
      else
         s = s2 ;
      endif
      C{j} = [xnext, min(y1next, y2next), s] ;
      j++ ;
   endwhile

endfunction