# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_service_from_delay.m
# input: AlphaL, AlphaU input bounds 
#        PhiL, PhiU delay bounds service 1
#       
#       
# output  SigmaL, SigmaU service bounds
#
# description: The function computes service bounds
#              from arrival and delay bounds
#
###########################################
function [SigmaL, SigmaU] = sla_service_from_delay(AlphaL, AlphaU, PhiL, PhiU)

   # The upper delay bound has to be met by the system with 
   # upper input process
   # Check first whetehr some quantitie sare infinite
   if (sla_is_infinite(PhiU))
      # infinite delay can reached by a server with zero speed
      SigmaL = {[0,0,0]} ;
   else
     if (sla_is_infinite(AlphaU))
       # we need a server with infinite speed 
       Sigmal = {[0,0,Inf]} ;
     else
       [GammaL, GammaI] = sla_output_bound(PhiU, AlphaU); 
       SigmaL = sla_min_deconv(GammaL, AlphaU) ;
     endif
   endif
   # The service lower servcie curve should be supper-additive
   # SigmaL = sla_max_closure(SigmaL) ;
   # For the lower delay bound we have to distinguish whether the 
   # the lower delay bound is zero, the lower arrival process is 
   # zero or both are non-zero
   if (sla_is_zero(PhiL))
      # no lower delay bound implies that the 
      # service can be arbirarily fast
      # SigmaU = {[0,inf,inf]} ;
      SigmaU = {[0,inf,inf]} ;
   else
      if (sla_is_zero(AlphaL))
         # the system has to meet for an arbitrarily small 
         # input the non-zero lower delay which requires an 
         # arbitrarily slow server
         printf("Cannot meet lower delay bound set service to lower service curve\n") ;
         SigmaU = SigmaL ;
      else
         [GammaU, GammaI] = sla_output_bound(PhiL, AlphaL) ;
	 # the servcie process equals the output process to reach the bound
   	 # SigmaU = GammaU ;
	 # this should be work as well
	 SigmaU = sla_min_deconv(GammaU, AlphaL) 
   	 # Now we have to check wehther the upper bound is
   	 # larger than the lower bound
   	 # If not, an error is printed and the upper bound
   	 # is modified, since artifical delays can always be
   	 # included
   	 if (not (sla_is_smaller_or_equal(SigmaL, SigmaU)))
     	    printf("Upper service curve too small, set to allowed minimum\n") ;
     	    SigmaU = sla_max(SigmaL, SigmaU) ;
   	 endif
      endif
   endif
   SigmaL = sla_reduce(SigmaL) ;
   SigmaU = sla_reduce(SigmaU) ;

endfunction