# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_test_bounding_curves.m
# input: AL, AU  matrices of the type
#        {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output true if both curves are consistent and curve AL is 
# stritcly smaller than curve AU 
# i.e. the slope of AL does not exceed the slope of AU
# and the y values of AU are large ror equal to the
# y values of AU.
#
###########################################
function ok = sla_test_bounding_curves(AL, AU)
#  ok =true ;
#  return ;
  if (not(sla_check_curve(AL)) || not(sla_check_curve(AU)) || not(sla_is_smaller_or_equal(AL, AU)))
    ok = false ;
    return ;
  endif
  iL = 1 ;
  iU = 1 ;
  if (AL{1}(3) > AU{1}(3))
    ok = false ;
    return ;
  else
    if (columns(AL) > 1)
      if (columns(AU) > 1)
        if (AL{2}(1) == AU{2}(1))
	  iL++ ;
	  iU++ ;
	else
	  if (AL{2}(1) < AU{2}(1))
	    iL++ ;
	  else
	    iU++ ;
	  endif
	endif
      else
        iL++ ;
      endif
    else
      if (columns(AU) > 1)
        iU++ ;
      endif
    endif
    ok = true ;
  endif
  while (iL < columns(AL) || iU < columns(AU))
    if (AL{iL}(3) > AU{iU}(3))
      ok = false ;
      return ;
    else
      if (columns(AL) > iL)
        if (columns(AU) > iU)
          if (AL{iL}(1) == AU{iU}(1))
	    iL++ ;
	    iU++ ;
	  else
	    if (AL{iL}(1) < AU{iU}(1))
	      iL++ ;
	    else
	      iU++ ;
	    endif
	  endif
        else
          iL++ ;
        endif
      else
        if (columns(AU) > iU)
          iU++ ;
        endif
      endif
    endif
  endwhile 
endfunction