# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_upper_or_delay.m
# input: P1U Upper delay curve of teh process
#	 b maximal size of load unit
#        p probability of choosing the service 
#          
# output PU upper delay curve
#
# description: The function computes the upper delay
#              curve for the service under parallel
#              or with size b and selection prob. p 
#
###########################################

function PU = sla_upper_or_delay(P1U, b, p)

  j = 1 ;
  found = false ;
  if (p == 0)
    PU = {[0, inf, inf]} ;
  else
    bo = b ;
    b = (1-p)*b ;
    # observe that b is now (1-p)b
    for (i=1:columns(P1U))
      if (P1U{i}(1) <= b && b > 0)
        if (i == 1)
          PU = {[P1U{1}(1), P1U{1}(2), P1U{1}(3)]} ;
        else
          PU{i} = P1U{i} ;
        endif
        j = i+1 ;
      else
        if (P1U{i}(1) > b && not(found) && i > 1)
          xval = bo ;
          yval = (xval-P1U{i-1}(1))*P1U{i-1}(3)+P1U{i-1}(2) ;
          sval = p*P1U{i}(3) ;
	  PU{j} = [xval, yval, sval] ;
          j++ ;
	  found = true ;
        else
	  xval = (P1U{i}(1)-bo)/p+bo ;
	  if (i > 1)
	    yval = PU{i-1}(2) + (xval-PU{i-1}(1))*PU{i-1}(3) ;
	  else
            yval =  P1U{1}(2) ;
	  endif
	  sval = p*P1U{i}(3) ;
	  if (j == 1)
	    PU = {[xval, yval, sval]} ;
	    found = true ;
	  else
	    PU{j} = [xval, yval, sval] ;
	  endif
          j++ ;
        endif
      endif
    endfor 
    # The case that the x value of the last segment is smaller than 
    # b still need to be considered
    if (P1U{columns(P1U)}(1) <= b)
        xval = bo ;
        yval = P1U{columns(P1U)}(2) + (bo-P1U{columns(P1U)}(2))*P1U{columns(P1U)}(3) ;
        sval = p * P1U{columns(P1U)}(3) ;
  	PU{j} = [xval, yval, sval] ;
    endif
    PU = sla_reduce(PU) ;
  endif
endfunction